@@ -3,6 +3,7 @@ package cmd
 import (
 	"context"
 	"fmt"
+	"io"
 	"log"
 	"os"
 	"os/exec"
@@ -20,14 +21,19 @@ import (
 	"github.com/cube2222/octosql/datasources/csv"
 	"github.com/cube2222/octosql/datasources/docs"
 	"github.com/cube2222/octosql/datasources/json"
+	"github.com/cube2222/octosql/datasources/lines"
 	"github.com/cube2222/octosql/datasources/plugins"
 	"github.com/cube2222/octosql/execution"
 	"github.com/cube2222/octosql/execution/nodes"
 	"github.com/cube2222/octosql/functions"
 	"github.com/cube2222/octosql/helpers/graph"
 	"github.com/cube2222/octosql/logical"
+	"github.com/cube2222/octosql/logs"
+	"github.com/cube2222/octosql/octosql"
 	"github.com/cube2222/octosql/optimizer"
 	"github.com/cube2222/octosql/outputs/batch"
+	"github.com/cube2222/octosql/outputs/eager"
+	"github.com/cube2222/octosql/outputs/formats"
 	"github.com/cube2222/octosql/outputs/stream"
 	"github.com/cube2222/octosql/parser"
 	"github.com/cube2222/octosql/parser/sqlparser"
@@ -64,6 +70,8 @@ octosql "SELECT * FROM plugins.plugins"`,
 		ctx := cmd.Context()
 		debug.SetGCPercent(1000)
 
+		logs.InitializeFileLogger()
+
 		pluginManager := &manager.PluginManager{}
 
 		pluginExecutor := executor.PluginExecutor{
@@ -143,6 +151,9 @@ octosql "SELECT * FROM plugins.plugins"`,
 		databases["docs"] = func() (physical.Database, error) {
 			return docs.Creator(ctx)
 		}
+		databases["lines"] = func() (physical.Database, error) {
+			return lines.Creator(ctx)
+		}
 
 		for _, metadata := range installedPlugins {
 			if _, ok := databases[metadata.Reference.Name]; ok {
@@ -164,15 +175,31 @@ octosql "SELECT * FROM plugins.plugins"`,
 				return db, nil
 			}
 		}
+
+		fileExtensionHandlers, err := pluginManager.GetFileExtensionHandlers()
+		if err != nil {
+			return fmt.Errorf("couldn't get file extension handlers: %w", err)
+		}
+		fileHandlers := map[string]func(name string, options map[string]string) (physical.DatasourceImplementation, physical.Schema, error){
+			"json": json.Creator,
+			"csv":  csv.Creator,
+		}
+		for ext, pluginName := range fileExtensionHandlers {
+			fileHandlers[ext] = func(name string, options map[string]string) (physical.DatasourceImplementation, physical.Schema, error) {
+				db, err := databases[pluginName]()
+				if err != nil {
+					return nil, physical.Schema{}, fmt.Errorf("couldn't get plugin %s database for plugin extensions %s: %w", pluginName, ext, err)
+				}
+				return db.GetTable(ctx, name, options)
+			}
+		}
+
 		env := physical.Environment{
 			Aggregates: aggregates.Aggregates,
 			Functions:  functions.FunctionMap(),
 			Datasources: &physical.DatasourceRepository{
-				Databases: databases,
-				FileHandlers: map[string]func(name string) (physical.DatasourceImplementation, physical.Schema, error){
-					"json": json.Creator,
-					"csv":  csv.Creator,
-				},
+				Databases:    databases,
+				FileHandlers: fileHandlers,
 			},
 			PhysicalConfig:  nil,
 			VariableContext: nil,
@@ -207,6 +234,51 @@ octosql "SELECT * FROM plugins.plugins"`,
 		}
 		reverseMapping := logical.ReverseMapping(mapping)
 
+		physicalOrderByExpressions := make([]physical.Expression, len(outputOptions.OrderByExpressions))
+		for i := range outputOptions.OrderByExpressions {
+			physicalExpr, err := typecheckExpr(ctx, outputOptions.OrderByExpressions[i], env.WithRecordSchema(physicalPlan.Schema), logical.Environment{
+				CommonTableExpressions: map[string]logical.CommonTableExpression{},
+				TableValuedFunctions:   tableValuedFunctions,
+				UniqueVariableNames: &logical.VariableMapping{
+					Mapping: mapping,
+				},
+				UniqueNameGenerator: uniqueNameGenerator,
+			})
+			if err != nil {
+				return fmt.Errorf("couldn't typecheck order by expression with index %d: %w", i, err)
+			}
+			physicalOrderByExpressions[i] = physicalExpr
+		}
+		if physicalPlan.Schema.NoRetractions && len(physicalOrderByExpressions) > 0 {
+			physicalPlan = physical.Node{
+				Schema:   physicalPlan.Schema,
+				NodeType: physical.NodeTypeOrderBy,
+				OrderBy: &physical.OrderBy{
+					Source:               physicalPlan,
+					Key:                  physicalOrderByExpressions,
+					DirectionMultipliers: logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
+				},
+			}
+			physicalOrderByExpressions = nil
+		}
+		if physicalPlan.Schema.NoRetractions && len(physicalOrderByExpressions) == 0 {
+			physicalPlan = physical.Node{
+				Schema:   physicalPlan.Schema,
+				NodeType: physical.NodeTypeLimit,
+				Limit: &physical.Limit{
+					Source: physicalPlan,
+					Limit: physical.Expression{
+						Type:           octosql.Int,
+						ExpressionType: physical.ExpressionTypeConstant,
+						Constant: &physical.Constant{
+							Value: octosql.NewInt(outputOptions.Limit),
+						},
+					},
+				},
+			}
+			outputOptions.Limit = 0
+		}
+
 		queryTelemetry := telemetry.GetQueryTelemetryData(physicalPlan, installedPlugins)
 
 		var executionPlan execution.Node
@@ -262,18 +334,7 @@ octosql "SELECT * FROM plugins.plugins"`,
 			}
 
 			orderByExpressions = make([]execution.Expression, len(outputOptions.OrderByExpressions))
-			for i := range outputOptions.OrderByExpressions {
-				physicalExpr, err := typecheckExpr(ctx, outputOptions.OrderByExpressions[i], env.WithRecordSchema(physicalPlan.Schema), logical.Environment{
-					CommonTableExpressions: map[string]logical.CommonTableExpression{},
-					TableValuedFunctions:   tableValuedFunctions,
-					UniqueVariableNames: &logical.VariableMapping{
-						Mapping: mapping,
-					},
-					UniqueNameGenerator: uniqueNameGenerator,
-				})
-				if err != nil {
-					return fmt.Errorf("couldn't typecheck order by expression with index %d: %w", i, err)
-				}
+			for i, physicalExpr := range physicalOrderByExpressions {
 				execExpr, err := physicalExpr.Materialize(ctx, env.WithRecordSchema(physicalPlan.Schema))
 				if err != nil {
 					return fmt.Errorf("couldn't materialize output order by expression with index %d: %v", i, err)
@@ -304,7 +365,9 @@ octosql "SELECT * FROM plugins.plugins"`,
 				logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
 				outputOptions.Limit,
 				outSchema,
-				batch.NewTableFormatter,
+				func(writer io.Writer) batch.Format {
+					return formats.NewTableFormatter(writer)
+				},
 				true,
 			)
 		case "batch_table":
@@ -314,19 +377,56 @@ octosql "SELECT * FROM plugins.plugins"`,
 				logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
 				outputOptions.Limit,
 				outSchema,
-				batch.NewTableFormatter,
+				func(writer io.Writer) batch.Format {
+					return formats.NewTableFormatter(writer)
+				},
 				false,
 			)
 		case "csv":
-			sink = batch.NewOutputPrinter(
-				executionPlan,
-				orderByExpressions,
-				logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
-				outputOptions.Limit,
-				outSchema,
-				batch.NewCSVFormatter,
-				false,
-			)
+			if !physicalPlan.Schema.NoRetractions {
+				sink = batch.NewOutputPrinter(
+					executionPlan,
+					orderByExpressions,
+					logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
+					outputOptions.Limit,
+					outSchema,
+					func(writer io.Writer) batch.Format {
+						return formats.NewCSVFormatter(writer)
+					},
+					false,
+				)
+			} else {
+				sink = eager.NewOutputPrinter(
+					executionPlan,
+					outSchema,
+					func(writer io.Writer) eager.Format {
+						return formats.NewCSVFormatter(writer)
+					},
+				)
+			}
+		case "json":
+			if !physicalPlan.Schema.NoRetractions {
+				sink = batch.NewOutputPrinter(
+					executionPlan,
+					orderByExpressions,
+					logical.DirectionsToMultipliers(outputOptions.OrderByDirections),
+					outputOptions.Limit,
+					outSchema,
+					func(writer io.Writer) batch.Format {
+						return formats.NewJSONFormatter(writer)
+					},
+					false,
+				)
+			} else {
+				sink = eager.NewOutputPrinter(
+					executionPlan,
+					outSchema,
+					func(writer io.Writer) eager.Format {
+						return formats.NewJSONFormatter(writer)
+					},
+				)
+			}
+
 		case "stream_native":
 			if len(orderByExpressions) > 0 {
 				executionPlan = nodes.NewBatchOrderBy(
@@ -372,7 +472,7 @@ func init() {
 	rootCmd.Flags().BoolVar(&describe, "describe", false, "Describe query output schema.")
 	rootCmd.Flags().IntVar(&explain, "explain", 0, "Describe query output schema.")
 	rootCmd.Flags().BoolVar(&optimize, "optimize", true, "Whether OctoSQL should optimize the query.")
-	rootCmd.Flags().StringVar(&output, "output", "live_table", "Output format to use. Available options are live_table, batch_table, csv and stream_native.")
+	rootCmd.Flags().StringVar(&output, "output", "live_table", "Output format to use. Available options are live_table, batch_table, csv, json and stream_native.")
 }
 
 func typecheckNode(ctx context.Context, node logical.Node, env physical.Environment, logicalEnv logical.Environment) (_ physical.Node, _ map[string]string, outErr error) {
