@@ -239,22 +239,27 @@ func ParseSelect(statement *sqlparser.Select, topmost bool) (logical.Node, *Outp
 
 	if statement.Limit != nil {
 		if !topmost {
+			limitExpr, err := ParseExpression(statement.Limit.Rowcount)
+			if err != nil {
+				return nil, nil, errors.Wrap(err, "couldn't parse limit")
+			}
+			root = logical.NewLimit(limitExpr, root)
 			return nil, nil, errors.Errorf("LIMIT in non-topmost expression not currently supported.")
-		}
+		} else {
+			l, ok := statement.Limit.Rowcount.(*sqlparser.SQLVal)
+			if !ok {
+				return nil, nil, errors.Errorf("LIMIT parameter must be constant, is: %+v", statement.Limit.Rowcount)
+			}
+			if l.Type != sqlparser.IntVal {
+				return nil, nil, errors.Errorf("LIMIT parameter must be Int constant, is: %+v", l.Type)
+			}
+			i, err := strconv.ParseInt(string(l.Val), 10, 64)
+			if err != nil {
+				return nil, nil, errors.Wrap(err, "LIMIT parameter must be Int constant, couldn't parse")
+			}
 
-		l, ok := statement.Limit.Rowcount.(*sqlparser.SQLVal)
-		if !ok {
-			return nil, nil, errors.Errorf("LIMIT parameter must be constant, is: %+v", statement.Limit.Rowcount)
+			outputOptions.Limit = int(i)
 		}
-		if l.Type != sqlparser.IntVal {
-			return nil, nil, errors.Errorf("LIMIT parameter must be Int constant, is: %+v", l.Type)
-		}
-		i, err := strconv.ParseInt(string(l.Val), 10, 64)
-		if err != nil {
-			return nil, nil, errors.Wrap(err, "LIMIT parameter must be Int constant, couldn't parse")
-		}
-
-		outputOptions.Limit = int(i)
 	}
 
 	return root, outputOptions, nil
@@ -322,6 +327,18 @@ func ParseAliasedTableExpression(expr *sqlparser.AliasedTableExpr) (logical.Node
 		if !subExpr.Qualifier.IsEmpty() {
 			name = fmt.Sprintf("%s.%s", subExpr.Qualifier.String(), name)
 		}
+		options := make(map[string]string)
+		if optionsStartIndex := strings.Index(name, "?"); optionsStartIndex != -1 {
+			optionsString := name[optionsStartIndex+1:]
+			name = name[:optionsStartIndex]
+			for i, option := range strings.Split(optionsString, "&") {
+				parts := strings.SplitN(option, "=", 2)
+				if len(parts) != 2 || parts[0] == "" || parts[1] == "" {
+					return nil, errors.Errorf("invalid option with index %d: %s", i, option)
+				}
+				options[parts[0]] = parts[1]
+			}
+		}
 		var alias string
 		if !expr.As.IsEmpty() {
 			alias = expr.As.String()
@@ -335,7 +352,7 @@ func ParseAliasedTableExpression(expr *sqlparser.AliasedTableExpr) (logical.Node
 				alias = alias[index+1:]
 			}
 		}
-		var out logical.Node = logical.NewDataSource(name, alias)
+		var out logical.Node = logical.NewDataSource(name, alias, options)
 		return out, nil
 
 	case *sqlparser.Subquery:
