@@ -26,19 +26,33 @@ type Node struct {
 	Requalifier         *Requalifier
 	TableValuedFunction *TableValuedFunction
 	Unnest              *Unnest
+	Limit               *Limit
 }
 
 type Schema struct {
 	Fields []SchemaField
 	// TimeField is -1 if not present.
-	TimeField int
+	TimeField     int
+	NoRetractions bool
 }
 
-func NewSchema(fields []SchemaField, timeField int) Schema {
-	return Schema{
+type SchemaOption func(schema *Schema)
+
+func NewSchema(fields []SchemaField, timeField int, options ...SchemaOption) Schema {
+	out := Schema{
 		Fields:    fields,
 		TimeField: timeField,
 	}
+	for _, opt := range options {
+		opt(&out)
+	}
+	return out
+}
+
+func WithNoRetractions(noRetractions bool) func(schema *Schema) {
+	return func(schema *Schema) {
+		schema.NoRetractions = noRetractions
+	}
 }
 
 type SchemaField struct {
@@ -60,6 +74,7 @@ const (
 	NodeTypeRequalifier
 	NodeTypeTableValuedFunction
 	NodeTypeUnnest
+	NodeTypeLimit
 )
 
 func (t NodeType) String() string {
@@ -86,6 +101,8 @@ func (t NodeType) String() string {
 		return "table_valued_function"
 	case NodeTypeUnnest:
 		return "unnest"
+	case NodeTypeLimit:
+		return "limit"
 	}
 	return "unknown"
 }
@@ -243,6 +260,11 @@ type Unnest struct {
 	Field  string
 }
 
+type Limit struct {
+	Source Node
+	Limit  Expression
+}
+
 func (node *Node) Materialize(ctx context.Context, env Environment) (execution.Node, error) {
 	switch node.NodeType {
 	case NodeTypeDatasource:
@@ -399,6 +421,17 @@ func (node *Node) Materialize(ctx context.Context, env Environment) (execution.N
 		}
 
 		return nodes.NewUnnest(source, index), nil
+
+	case NodeTypeLimit:
+		source, err := node.Limit.Source.Materialize(ctx, env)
+		if err != nil {
+			return nil, fmt.Errorf("couldn't materialize limit source: %w", err)
+		}
+		limit, err := node.Limit.Limit.Materialize(ctx, env)
+		if err != nil {
+			return nil, fmt.Errorf("couldn't materialize limit expression: %w", err)
+		}
+		return nodes.NewLimit(source, limit), nil
 	}
 
 	panic(fmt.Sprintf("unexhaustive node type match: %d", node.NodeType))
